@@ -3,7 +3,6 @@
 /*			    ACT Command Support 				*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: Object_spt.c 1490 2019-07-26 21:13:22Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,36 +54,36 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2023				*/
 /*										*/
 /********************************************************************************/
 
-/* 7.8	ACT Support (ACT_spt.c) */
-/* 7.8.1	Introduction */
-/* This code implements the ACT update code. It does not use a mutex. This code uses a platform
-   service (_plat__ACT_UpdateCounter()) that returns false if the update is not accepted. If this
-   occurs, then TPM_RC_RETRY should be sent to the caller so that they can retry the operation
-   later. The implementation of this is platform dependent but the reference uses a simple flag to
-   indicate that an update is pending and the only process that can clear that flag is the process
-   that does the actual update. */
-
-/* 7.8.2	Includes */
+//** Introduction
+// This code implements the ACT update code. It does not use a mutex. This code uses
+// a platform service (_plat__ACT_UpdateCounter()) that returns 'false' if the update
+// is not accepted. If this occurs, then TPM_RC_RETRY should be sent to the caller so
+// that they can retry the operation later. The implementation of this is platform
+// dependent but the reference uses a simple flag to indicate that an update is
+// pending and the only process that can clear that flag is the process that does the
+// actual update.
 
+//** Includes
 #include "Tpm.h"
 #include "ACT_spt_fp.h"
-#include "Platform_fp.h"
-#include "PlatformACT_fp.h"		/* added kgold */
+// TODO_RENAME_INC_FOLDER:platform_interface refers to the TPM_CoreLib platform interface
+#include "tpm_to_platform_interface.h"
 
-/* 7.8.3	Functions */
-/* 7.8.3.1	_ActResume() */
-/* This function does the resume processing for an ACT. It updates the saved count and turns
-   signaling back on if necessary. */
+//** Functions
+
+#if 1 // ACT_SUPPORT			// libtpms: FIXME
+
+//*** _ActResume()
+// This function does the resume processing for an ACT. It updates the saved count
+// and turns signaling back on if necessary.
 #ifndef __ACT_DISABLED	// libtpms added
-static void
-_ActResume(
-	   UINT32              act,            //IN: the act number
-	   ACT_STATE          *actData         //IN: pointer to the saved ACT data
-	   )
+static void _ActResume(UINT32     act,     //IN: the act number
+		       ACT_STATE* actData  //IN: pointer to the saved ACT data
+		       )
 {
     // If the act was non-zero, then restore the counter value.
     if(actData->remaining > 0)
@@ -94,20 +93,18 @@ _ActResume(
 	_plat__ACT_SetSignaled(act, TRUE);
 }
 #endif			// libtpms added
-/* 7.8.3.2	ActStartup() */
-/* This function is called by TPM2_Startup() to initialize the ACT counter values. */
-BOOL
-ActStartup(
-	   STARTUP_TYPE        type
-	   )
+
+//*** ActStartup()
+// This function is called by TPM2_Startup() to initialize the ACT counter values.
+BOOL ActStartup(STARTUP_TYPE type)
 {
     // Reset all the ACT hardware
     _plat__ACT_Initialize();
-
+    
     // If this not a cold start, copy all the current 'signaled' settings to
     // 'preservedSignaled'.
 #ifndef __ACT_DISABLED	// libtpms added
-    if (g_powerWasLost)
+    if(g_powerWasLost)
 	go.preservedSignaled = 0;
     else
 	go.preservedSignaled |= go.signaledACT;
@@ -120,17 +117,15 @@ ActStartup(
 #ifndef __ACT_DISABLED	// libtpms added
 	    go.signaledACT = 0;
 #endif			// libtpms added
-#define CLEAR_ACT_POLICY(N)						\
-	    go.ACT_##N.hashAlg = TPM_ALG_NULL;				\
+#  define CLEAR_ACT_POLICY(N)					      \
+	    go.ACT_##N.hashAlg           = TPM_ALG_NULL;	      \
 	    go.ACT_##N.authPolicy.b.size = 0;
-	    
 	    FOR_EACH_ACT(CLEAR_ACT_POLICY)
-		
 		}
     else
 	{
 	    // Resume each of the implemented ACT
-#define RESUME_ACT(N)   _ActResume(0x##N, &go.ACT_##N);
+#  define RESUME_ACT(N) _ActResume(0x##N, &go.ACT_##N);
 	    
 	    FOR_EACH_ACT(RESUME_ACT)
 		}
@@ -140,15 +135,12 @@ ActStartup(
     _plat__ACT_EnableTicks(TRUE);
     return TRUE;
 }
-/* 7.8.3.3	_ActSaveState() */
-/* Get the counter state and the signaled state for an ACT. If the ACT has not been updated since
-   the last time it was saved, then divide the count by 2. */
-#ifndef __ACT_DISABLED	// libtpms added
-static void
-_ActSaveState(
-	      UINT32              act,
-	      P_ACT_STATE         actData
-	      )
+
+#ifndef __ACT_DISABLED // libtpms added
+//*** _ActSaveState()
+// Get the counter state and the signaled state for an ACT. If the ACT has not been
+// updated since the last time it was saved, then divide the count by 2.
+static void _ActSaveState(UINT32 act, P_ACT_STATE actData)
 {
     actData->remaining = _plat__ACT_GetRemaining(act);
     // If the ACT hasn't been updated since the last startup, then it should be
@@ -162,24 +154,21 @@ _ActSaveState(
     if(_plat__ACT_GetSignaled(act))
 	go.signaledACT |= (1 << act);
 }
-/* 7.8.3.4	ActGetSignaled() */
-/* This function returns the state of the signaled flag associated with an ACT. */
-BOOL
-ActGetSignaled(
-	       TPM_RH              actHandle
-	       )
+
+//*** ActGetSignaled()
+// This function returns the state of the signaled flag associated with an ACT.
+BOOL ActGetSignaled(TPM_RH actHandle)
 {
-    UINT32              act = actHandle - TPM_RH_ACT_0;
+    UINT32 act = actHandle - TPM_RH_ACT_0;
     //
     return _plat__ACT_GetSignaled(act);
 }
 #endif			// libtpms added
-/* 7.8.3.5	ActShutdown() */
-/* This function saves the current state of the counters */
-BOOL
-ActShutdown(
-	    TPM_SU              state       //IN: the type of the shutdown.
-	    )
+
+//***ActShutdown()
+// This function saves the current state of the counters
+BOOL ActShutdown(TPM_SU state  //IN: the type of the shutdown.
+		 )
 {
     // if this is not shutdown state, then the only type of startup is TPM_RESTART
     // so the timer values will be cleared. If this is shutdown state, get the current
@@ -193,26 +182,24 @@ ActShutdown(
 	    go.signaledACT = 0;
 #endif				// libtpms added
 	    // Get the current count and the signaled state
-#define SAVE_ACT_STATE(N) _ActSaveState(0x##N, &go.ACT_##N);
+#  define SAVE_ACT_STATE(N) _ActSaveState(0x##N, &go.ACT_##N);
 	    
 	    FOR_EACH_ACT(SAVE_ACT_STATE);
 	}
     return TRUE;
 }
-/* 7.8.3.6	ActIsImplemented() */
-/* This function determines if an ACT is implemented in both the TPM and the platform code. */
-BOOL
-ActIsImplemented(
-		 UINT32          act
-		 )
+
+//*** ActIsImplemented()
+// This function determines if an ACT is implemented in both the TPM and the platform
+// code.
+BOOL ActIsImplemented(UINT32 act)
 {
-#define CASE_ACT_
     // This switch accounts for the TPM implemented values.
     switch(act)
 	{
 #ifndef __ACT_DISABLED	// libtpms added
 	    FOR_EACH_ACT(CASE_ACT_NUMBER)
-		// This ensures that the platorm implemented the values implemented by
+		// This ensures that the platform implements the values implemented by
 		// the TPM
 		return _plat__ACT_GetImplemented(act);
 #endif			// libtpms added
@@ -221,18 +208,18 @@ ActIsImplemented(
 	}
     return FALSE;
 }
-/* 7.8.3.7	ActCounterUpdate() */
-/* This function updates the ACT counter. If the counter already has a pending update, it returns
-   TPM_RC_RETRY so that the update can be tried again later. */
+
 #if CC_ACT_SetTimeout	// libtpms added
+//***ActCounterUpdate()
+// This function updates the ACT counter. If the counter already has a pending update,
+// it returns TPM_RC_RETRY so that the update can be tried again later.
 TPM_RC
-ActCounterUpdate(
-		 TPM_RH          handle,         //IN: the handle of the act
-		 UINT32          newValue        //IN: the value to set in the ACT
+ActCounterUpdate(TPM_RH handle,   //IN: the handle of the act
+		 UINT32 newValue  //IN: the value to set in the ACT
 		 )
 {
-    UINT32          act;
-    TPM_RC          result;
+    UINT32 act;
+    TPM_RC result;
     //
     act = handle - TPM_RH_ACT_0;
     // This should never fail, but...
@@ -240,7 +227,8 @@ ActCounterUpdate(
 	result = TPM_RC_VALUE;
     else
 	{
-	    // Will need to clear orderly so fail if we are orderly and NV is not available
+	    // Will need to clear orderly so fail if we are orderly and NV is
+	    // not available
 	    if(NV_IS_ORDERLY)
 		RETURN_IF_NV_IS_NOT_AVAILABLE;
 	    // if the attempt to update the counter fails, it means that there is an
@@ -248,32 +236,32 @@ ActCounterUpdate(
 	    if(!_plat__ACT_UpdateCounter(act, newValue))
 		result = TPM_RC_RETRY;
 	    else
-	        {
-	            // Indicate that the ACT has been updated since last TPM2_Startup().
-	            s_ActUpdated |= (UINT16)(1 << act);
-
+		{
+		    // Indicate that the ACT has been updated since last TPM2_Startup().
+		    s_ActUpdated |= (UINT16)(1 << act);
+		    
 		    // Clear the preservedSignaled attribute.
 		    go.preservedSignaled &= ~((UINT16)(1 << act));
 		    
-	            // Need to clear the orderly flag
-	            g_clearOrderly = TRUE;
+		    // Need to clear the orderly flag
+		    g_clearOrderly = TRUE;
 		    
-	            result = TPM_RC_SUCCESS;
-	        }
+		    result         = TPM_RC_SUCCESS;
+		}
 	}
     return result;
 }
 #endif		// libtpms added
-/* 7.8.3.8	ActGetCapabilityData() */
-/* This function returns the list of ACT data */
-/* Return Value	Meaning */
-/* YES	if more ACT data is available */
-/* NO	if no more ACT data to */
+
+//*** ActGetCapabilityData()
+// This function returns the list of ACT data
+//  Return Type: TPMI_YES_NO
+//      YES             if more ACT data is available
+//      NO              if no more ACT data to
 TPMI_YES_NO
-ActGetCapabilityData(
-		     TPM_HANDLE       actHandle,     // IN: the handle for the starting ACT
-		     UINT32           maxCount,      // IN: maximum allowed return values
-		     TPML_ACT_DATA   *actList        // OUT: ACT data list
+ActGetCapabilityData(TPM_HANDLE     actHandle,  // IN: the handle for the starting ACT
+		     UINT32         maxCount,   // IN: maximum allowed return values
+		     TPML_ACT_DATA* actList     // OUT: ACT data list
 		     )
 {
     // Initialize output property list
@@ -288,20 +276,24 @@ ActGetCapabilityData(
     // Scan the ACT data from the starting ACT
     for(; actHandle <= TPM_RH_ACT_F; actHandle++)
 	{
-	    UINT32          act = actHandle - TPM_RH_ACT_0;
+	    UINT32 act = actHandle - TPM_RH_ACT_0;
 	    if(actList->count < maxCount)
-	        {
-	            if(ActIsImplemented(act))
+		{
+		    if(ActIsImplemented(act))
 			{
-			    TPMS_ACT_DATA    *actData = &actList->actData[actList->count];
+			    TPMS_ACT_DATA* actData = &actList->actData[actList->count];
 			    //
 			    memset(&actData->attributes, 0, sizeof(actData->attributes));
-			    actData->handle = actHandle;
+			    actData->handle  = actHandle;
 			    actData->timeout = _plat__ACT_GetRemaining(act);
-			    if (_plat__ACT_GetSignaled(act))
+			    if(_plat__ACT_GetSignaled(act))
 				SET_ATTRIBUTE(actData->attributes, TPMA_ACT, signaled);
 			    else
 				CLEAR_ATTRIBUTE(actData->attributes, TPMA_ACT, signaled);
+#ifndef __ACT_DISABLED	// libtpms added
+			    if(go.preservedSignaled & (1 << act))
+				SET_ATTRIBUTE(actData->attributes, TPMA_ACT, preserveSignaled);
+#endif			// libtpms added
 			    actList->count++;
 			}
 		}
@@ -315,3 +307,31 @@ ActGetCapabilityData(
     // was filled and there are no more ACT values to return
     return NO;
 }
+
+#ifndef __ACT_DISABLED		// libtpms: added
+//*** ActGetOneCapability()
+// This function returns an ACT's capability, if present.
+BOOL ActGetOneCapability(TPM_HANDLE     actHandle,  // IN: the handle for the ACT
+			 TPMS_ACT_DATA* actData     // OUT: ACT data
+			 )
+{
+    UINT32 act = actHandle - TPM_RH_ACT_0;
+    
+    if(ActIsImplemented(actHandle - TPM_RH_ACT_0))
+	{
+	    memset(&actData->attributes, 0, sizeof(actData->attributes));
+	    actData->handle  = actHandle;
+	    actData->timeout = _plat__ACT_GetRemaining(act);
+	    if(_plat__ACT_GetSignaled(act))
+		SET_ATTRIBUTE(actData->attributes, TPMA_ACT, signaled);
+	    else
+		CLEAR_ATTRIBUTE(actData->attributes, TPMA_ACT, signaled);
+	    if(go.preservedSignaled & (1 << act))
+		SET_ATTRIBUTE(actData->attributes, TPMA_ACT, preserveSignaled);
+	    return TRUE;
+	}
+    return FALSE;
+}
+#endif				// libtpms: added
+
+#endif  // ACT_SUPPORT
