@@ -3,7 +3,6 @@
 /*			     Hierarchy Commands					*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: HierarchyCommands.c 1519 2019-11-15 20:43:51Z kgoldman $	*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,7 +54,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2019				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2023				*/
 /*										*/
 /********************************************************************************/
 
@@ -73,6 +72,7 @@ TPM2_CreatePrimary(
     DRBG_STATE           rand;
     OBJECT              *newObject;
     TPM2B_NAME           name;
+    TPM2B_SEED     	primary_seed;
     // Input Validation
     // Will need a place to put the result
     newObject = FindEmptyObjectSlot(&out->objectHandle);
@@ -80,18 +80,19 @@ TPM2_CreatePrimary(
 	return TPM_RC_OBJECT_MEMORY;
     // Get the address of the public area in the new object
     // (this is just to save typing)
-    publicArea = &newObject->publicArea;
+    publicArea  = &newObject->publicArea;
+
     *publicArea = in->inPublic.publicArea;
+
     // Check attributes in input public area. CreateChecks() checks the things that
     // are unique to creation and then validates the attributes and values that are
     // common to create and load.
-    result = CreateChecks(NULL, publicArea,
-			  in->inSensitive.sensitive.data.t.size);
+    result = CreateChecks(
+			  NULL, in->primaryHandle, publicArea, in->inSensitive.sensitive.data.t.size);
     if(result != TPM_RC_SUCCESS)
 	return RcSafeAddToResult(result, RC_CreatePrimary_inPublic);
     // Validate the sensitive area values
-    if(!AdjustAuthSize(&in->inSensitive.sensitive.userAuth,
-		       publicArea->nameAlg))
+    if(!AdjustAuthSize(&in->inSensitive.sensitive.userAuth, publicArea->nameAlg))
 	return TPM_RCS_SIZE + RC_CreatePrimary_inSensitive;
     // Command output
     // Compute the name using out->name as a scratch area (this is not the value
@@ -99,33 +100,54 @@ TPM2_CreatePrimary(
     // used as a random number generator during the object creation.
     // The caller does not know the seed values so the actual name does not have
     // to be over the input, it can be over the unmarshaled structure.
-    result = DRBG_InstantiateSeeded(&rand,
-				    &HierarchyGetPrimarySeed(in->primaryHandle)->b,
-				    PRIMARY_OBJECT_CREATION,
-				    (TPM2B *)PublicMarshalAndComputeName(publicArea, &name),
-				    &in->inSensitive.sensitive.data.b,
-				    HierarchyGetPrimarySeedCompatLevel(in->primaryHandle)); // libtpms added
-    if (result == TPM_RC_SUCCESS)
+
+    result = HierarchyGetPrimarySeed(in->primaryHandle, &primary_seed);
+    if(result != TPM_RC_SUCCESS)
+	return result;
+
+    result =
+	DRBG_InstantiateSeeded(&rand,
+			       &primary_seed.b,
+			       PRIMARY_OBJECT_CREATION,
+			       (TPM2B*)PublicMarshalAndComputeName(publicArea, &name),
+			       &in->inSensitive.sensitive.data.b,
+			       HierarchyGetPrimarySeedCompatLevel(in->primaryHandle)); // libtpms added
+    MemorySet(primary_seed.b.buffer, 0, primary_seed.b.size);
+
+    if(result == TPM_RC_SUCCESS)
 	{
 	    newObject->attributes.primary = SET;
-	    if(in->primaryHandle == TPM_RH_ENDORSEMENT)
+	    if(HierarchyNormalizeHandle(in->primaryHandle) == TPM_RH_ENDORSEMENT)
 		newObject->attributes.epsHierarchy = SET;
+
 	    // Create the primary object.
-	    result = CryptCreateObject(newObject, &in->inSensitive.sensitive,
-				       (RAND_STATE *)&rand);
+	    result = CryptCreateObject(
+				       newObject, &in->inSensitive.sensitive, (RAND_STATE*)&rand);
+	    DRBG_Uninstantiate(&rand);
 	}
     if(result != TPM_RC_SUCCESS)
 	return result;
+
     // Set the publicArea and name from the computed values
     out->outPublic.publicArea = newObject->publicArea;
-    out->name = newObject->name;
+    out->name                 = newObject->name;
+
     // Fill in creation data
-    FillInCreationData(in->primaryHandle, publicArea->nameAlg,
-		       &in->creationPCR, &in->outsideInfo, &out->creationData,
+    FillInCreationData(in->primaryHandle,
+		       publicArea->nameAlg,
+		       &in->creationPCR,
+		       &in->outsideInfo,
+		       &out->creationData,
 		       &out->creationHash);
+
     // Compute creation ticket
-    TicketComputeCreation(EntityGetHierarchy(in->primaryHandle), &out->name,
-			  &out->creationHash, &out->creationTicket);
+    result = TicketComputeCreation(EntityGetHierarchy(in->primaryHandle),
+				   &out->name,
+				   &out->creationHash,
+				   &out->creationTicket);
+    if(result != TPM_RC_SUCCESS)
+	return result;
+
     // Set the remaining attributes for a loaded object
     ObjectSetLoadedAttributes(newObject, in->primaryHandle,
                               HierarchyGetPrimarySeedCompatLevel(in->primaryHandle)); // libtpms added
@@ -240,7 +262,7 @@ TPM2_SetPrimaryPolicy(
     if(in->authPolicy.t.size != CryptHashGetDigestSize(in->hashAlg))
 	return TPM_RCS_SIZE + RC_SetPrimaryPolicy_authPolicy;
     // The command need NV update for OWNER and ENDORSEMENT hierarchy, and
-    // might need orderlyState update for PLATFORM hierarchy.
+    // might need orderlyState update for PLATFROM hierarchy.
     // Check if NV is available.  A TPM_RC_NV_UNAVAILABLE or TPM_RC_NV_RATE
     // error may be returned at this point
     RETURN_IF_NV_IS_NOT_AVAILABLE;
@@ -279,9 +301,9 @@ TPM2_SetPrimaryPolicy(
 	      go.ACT_##N.authPolicy = in->authPolicy;			\
 	      g_clearOrderly = TRUE;					\
 	      break;
-	    
+
 	    FOR_EACH_ACT(SET_ACT_POLICY)
-	    
+
 	  default:
 	    FAIL(FATAL_ERROR_INTERNAL);
 	    break;
@@ -306,7 +328,7 @@ TPM2_ChangePPS(
     // Internal Data Update
     // Reset platform hierarchy seed from RNG
     CryptRandomGenerate(sizeof(gp.PPSeed.t.buffer), gp.PPSeed.t.buffer);
-    gp.PPSeedCompatLevel = SEED_COMPAT_LEVEL_LAST; // libtpms added
+    gp.PPSeedCompatLevel = RuntimeProfileGetSeedCompatLevel(); // libtpms added
     // Create a new phProof value from RNG to prevent the saved platform
     // hierarchy contexts being loaded
     CryptRandomGenerate(sizeof(gp.phProof.t.buffer), gp.phProof.t.buffer);
@@ -352,7 +374,7 @@ TPM2_ChangeEPS(
     // Internal Data Update
     // Reset endorsement hierarchy seed from RNG
     CryptRandomGenerate(sizeof(gp.EPSeed.t.buffer), gp.EPSeed.t.buffer);
-    gp.EPSeedCompatLevel = SEED_COMPAT_LEVEL_LAST; // libtpms added
+    gp.EPSeedCompatLevel = RuntimeProfileGetSeedCompatLevel(); // libtpms added
     // Create new ehProof value from RNG
     CryptRandomGenerate(sizeof(gp.ehProof.t.buffer), gp.ehProof.t.buffer);
     // Enable endorsement hierarchy
@@ -401,7 +423,7 @@ TPM2_Clear(
     // Internal Data Update
     // Reset storage hierarchy seed from RNG
     CryptRandomGenerate(sizeof(gp.SPSeed.t.buffer), gp.SPSeed.t.buffer);
-    gp.SPSeedCompatLevel = SEED_COMPAT_LEVEL_LAST; // libtpms added
+    gp.SPSeedCompatLevel = RuntimeProfileGetSeedCompatLevel(); // libtpms added
     // Create new shProof and ehProof value from RNG
     CryptRandomGenerate(sizeof(gp.shProof.t.buffer), gp.shProof.t.buffer);
     CryptRandomGenerate(sizeof(gp.ehProof.t.buffer), gp.ehProof.t.buffer);
