@@ -3,7 +3,6 @@
 /*			     Symmetric block cipher modes			*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: CryptSym.c 1658 2021-01-22 23:14:01Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,7 +54,7 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2021				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2024				*/
 /*										*/
 /********************************************************************************/
 
@@ -175,14 +174,16 @@ CryptSymmetricEncrypt(
     tpmCryptKeySchedule_t        keySchedule;
     INT16                blockSize;
     TpmCryptSetSymKeyCall_t        encrypt;
+    TpmCryptSymFinal_t             final; // libtpms added
     BYTE                *iv;
     BYTE                 defaultIv[MAX_SYM_BLOCK_SIZE] = {0};
     //
-    memset(&keySchedule, 0, sizeof(keySchedule));	// libtpms added; coverity
     pAssert(dOut != NULL && key != NULL && dIn != NULL);
+    memset((void *)&keySchedule, 0, sizeof(keySchedule));	/* silence false positive; coverity */
+    memset(tmp, 0, sizeof(tmp));
     if(dSize == 0)
 	return TPM_RC_SUCCESS;
-    TEST(algorithm);
+    TPM_DO_SELF_TEST(algorithm);
     blockSize = CryptGetSymmetricBlockSize(algorithm, keySizeInBits);
     if(blockSize == 0)
 	return TPM_RC_FAILURE;
@@ -294,8 +295,12 @@ CryptSymmetricEncrypt(
 	    break;
 #endif
 	  default:
+	    if (final)			// libtpms added begin
+		FINAL(&keySchedule);	// libtpms added end
 	    return TPM_RC_FAILURE;
 	}
+    if (final)				// libtpms added begin
+	FINAL(&keySchedule);		// libtpms added end
     return TPM_RC_SUCCESS;
 }
 /* 10.2.20.5.1 CryptSymmetricDecrypt() */
@@ -326,9 +331,11 @@ CryptSymmetricDecrypt(
     BYTE                *iv;
     TpmCryptSetSymKeyCall_t        encrypt;
     TpmCryptSetSymKeyCall_t        decrypt;
+    TpmCryptSymFinal_t             final; /* libtpms added */
     BYTE                 defaultIv[MAX_SYM_BLOCK_SIZE] = {0};
 
-    memset(&keySchedule, 0, sizeof(keySchedule));	// libtpms added; coverity
+    memset((void *)&keySchedule, 0, sizeof(keySchedule));	/* silence false positive; coverity */
+    memset(tmp, 0, sizeof(tmp));
     // These are used but the compiler can't tell because they are initialized
     // in case statements and it can't tell if they are always initialized
     // when needed, so... Comment these out if the compiler can tell or doesn't
@@ -338,7 +345,7 @@ CryptSymmetricDecrypt(
     pAssert(dOut != NULL && key != NULL && dIn != NULL);
     if(dSize == 0)
 	return TPM_RC_SUCCESS;
-    TEST(algorithm);
+    TPM_DO_SELF_TEST(algorithm);
     blockSize = CryptGetSymmetricBlockSize(algorithm, keySizeInBits);
     if(blockSize == 0)
 	return TPM_RC_FAILURE;
@@ -466,8 +473,12 @@ CryptSymmetricDecrypt(
 	    break;
 #endif
 	  default:
+	    if (final)			/* libtpms added begin */
+		FINAL(&keySchedule);	/* libtpms added end */
 	    return TPM_RC_FAILURE;
 	}
+    if (final)				/* libtpms added begin */
+	FINAL(&keySchedule);		/* libtpms added end */
     return TPM_RC_SUCCESS;
 }
 
@@ -530,7 +541,7 @@ CryptSymmetricEncrypt(
     INT16                blockSize;
     BYTE                *iv;
     BYTE                 defaultIv[MAX_SYM_BLOCK_SIZE] = {0};
-    evpfunc              evpfn;
+    const EVP_CIPHER    *evp_cipher;
     EVP_CIPHER_CTX      *ctx = NULL;
     int                  outlen1 = 0;
     int                  outlen2 = 0;
@@ -545,7 +556,7 @@ CryptSymmetricEncrypt(
     pAssert(dOut != NULL && key != NULL && dIn != NULL);
     if(dSize == 0)
 	return TPM_RC_SUCCESS;
-    TEST(algorithm);
+    TPM_DO_SELF_TEST(algorithm);
     blockSize = CryptGetSymmetricBlockSize(algorithm, keySizeInBits);
     if(blockSize == 0)
 	return TPM_RC_FAILURE;
@@ -570,9 +581,9 @@ CryptSymmetricEncrypt(
 		return TPM_RC_SIZE;
         }
 
-    evpfn = GetEVPCipher(algorithm, keySizeInBits, mode, key,
-                         keyToUse, &keyToUseLen);
-    if (evpfn == NULL)
+    evp_cipher = GetEVPCipher(algorithm, keySizeInBits, mode, key,
+                              keyToUse, &keyToUseLen);
+    if (evp_cipher == NULL)
         return TPM_RC_FAILURE;
 
     if (dIn == dOut) {
@@ -580,7 +591,7 @@ CryptSymmetricEncrypt(
         buffersize = TPM2_ROUNDUP(dSize, blockSize);
         buffer = malloc(buffersize);
         if (buffer == NULL)
-            ERROR_RETURN(TPM_RC_FAILURE);
+            ERROR_EXIT(TPM_RC_FAILURE);
 
         pOut = buffer;
     }
@@ -589,29 +600,30 @@ CryptSymmetricEncrypt(
     if (algorithm == TPM_ALG_TDES && mode == TPM_ALG_CTR) {
         TDES_CTR(keyToUse, keyToUseLen * 8, dSize, dIn, iv, pOut, blockSize);
         outlen1 = dSize;
-        ERROR_RETURN(TPM_RC_SUCCESS);
+        ERROR_EXIT(TPM_RC_SUCCESS);
     }
 #endif
 
     ctx = EVP_CIPHER_CTX_new();
     if (!ctx ||
-        EVP_EncryptInit_ex(ctx, evpfn(), NULL, keyToUse, iv) != 1 ||
+        EVP_EncryptInit_ex(ctx, evp_cipher, NULL, keyToUse, iv) != 1 ||
         EVP_CIPHER_CTX_set_padding(ctx, 0) != 1 ||
         EVP_EncryptUpdate(ctx, pOut, &outlen1, dIn, dSize) != 1)
-        ERROR_RETURN(TPM_RC_FAILURE);
+        ERROR_EXIT(TPM_RC_FAILURE);
 
     pAssert(outlen1 <= dSize || dSize >= outlen1 + blockSize);
 
     if (EVP_EncryptFinal_ex(ctx, pOut + outlen1, &outlen2) != 1)
-        ERROR_RETURN(TPM_RC_FAILURE);
+        ERROR_EXIT(TPM_RC_FAILURE);
 
     if (ivInOut) {
         ivLen = EVP_CIPHER_CTX_iv_length(ctx);
         if (ivLen < 0 || (size_t)ivLen > sizeof(ivInOut->t.buffer))
-            ERROR_RETURN(TPM_RC_FAILURE);
+            ERROR_EXIT(TPM_RC_FAILURE);
 
         ivInOut->t.size = ivLen;
-        memcpy(ivInOut->t.buffer, EVP_CIPHER_CTX_iv(ctx), ivInOut->t.size);
+        if (ivLen > 0 && DoEVPGetIV(ctx, ivInOut->t.buffer, ivInOut->t.size))
+            ERROR_EXIT(TPM_RC_FAILURE);
     }
  Exit:
     if (retVal == TPM_RC_SUCCESS && pOut != dOut)
@@ -645,7 +657,7 @@ CryptSymmetricDecrypt(
     INT16                blockSize;
     BYTE                *iv;
     BYTE                 defaultIv[MAX_SYM_BLOCK_SIZE] = {0};
-    evpfunc              evpfn;
+    const EVP_CIPHER    *evp_cipher;
     EVP_CIPHER_CTX      *ctx = NULL;
     int                  outlen1 = 0;
     int                  outlen2 = 0;
@@ -663,7 +675,7 @@ CryptSymmetricDecrypt(
     pAssert(dOut != NULL && key != NULL && dIn != NULL);
     if(dSize == 0)
 	return TPM_RC_SUCCESS;
-    TEST(algorithm);
+    TPM_DO_SELF_TEST(algorithm);
     blockSize = CryptGetSymmetricBlockSize(algorithm, keySizeInBits);
     if(blockSize == 0)
 	return TPM_RC_FAILURE;
@@ -693,47 +705,48 @@ CryptSymmetricDecrypt(
 	    break;
 	}
 
-    evpfn = GetEVPCipher(algorithm, keySizeInBits, mode, key,
-                         keyToUse, &keyToUseLen);
-    if (evpfn ==  NULL)
+    evp_cipher = GetEVPCipher(algorithm, keySizeInBits, mode, key,
+                              keyToUse, &keyToUseLen);
+    if (evp_cipher == NULL)
         return TPM_RC_FAILURE;
 
     /* a buffer with a 'safety margin' for EVP_DecryptUpdate */
     buffersize = TPM2_ROUNDUP(dSize + blockSize, blockSize);
     buffer = malloc(buffersize);
     if (buffer == NULL)
-        ERROR_RETURN(TPM_RC_FAILURE);
+        ERROR_EXIT(TPM_RC_FAILURE);
 
 #if ALG_TDES && ALG_CTR
     if (algorithm == TPM_ALG_TDES && mode == TPM_ALG_CTR) {
         TDES_CTR(keyToUse, keyToUseLen * 8, dSize, dIn, iv, buffer, blockSize);
         outlen1 = dSize;
-        ERROR_RETURN(TPM_RC_SUCCESS);
+        ERROR_EXIT(TPM_RC_SUCCESS);
     }
 #endif
 
     ctx = EVP_CIPHER_CTX_new();
     if (!ctx ||
-        EVP_DecryptInit_ex(ctx, evpfn(), NULL, keyToUse, iv) != 1 ||
+        EVP_DecryptInit_ex(ctx, evp_cipher, NULL, keyToUse, iv) != 1 ||
         EVP_CIPHER_CTX_set_padding(ctx, 0) != 1 ||
         EVP_DecryptUpdate(ctx, buffer, &outlen1, dIn, dSize) != 1)
-        ERROR_RETURN(TPM_RC_FAILURE);
+        ERROR_EXIT(TPM_RC_FAILURE);
 
     pAssert((int)buffersize >= outlen1);
 
     if ((int)buffersize <= outlen1 /* coverity */ ||
         EVP_DecryptFinal(ctx, &buffer[outlen1], &outlen2) != 1)
-        ERROR_RETURN(TPM_RC_FAILURE);
+        ERROR_EXIT(TPM_RC_FAILURE);
 
     pAssert((int)buffersize >= outlen1 + outlen2);
 
     if (ivInOut) {
         ivLen = EVP_CIPHER_CTX_iv_length(ctx);
         if (ivLen < 0 || (size_t)ivLen > sizeof(ivInOut->t.buffer))
-            ERROR_RETURN(TPM_RC_FAILURE);
+            ERROR_EXIT(TPM_RC_FAILURE);
 
         ivInOut->t.size = ivLen;
-        memcpy(ivInOut->t.buffer, EVP_CIPHER_CTX_iv(ctx), ivInOut->t.size);
+        if (ivLen > 0 && DoEVPGetIV(ctx, ivInOut->t.buffer, ivInOut->t.size))
+            ERROR_EXIT(TPM_RC_FAILURE);
     }
 
  Exit:
