@@ -71,15 +71,20 @@ OpenSSLCryptGenerateKeyDes(
                            TPMT_SENSITIVE *sensitive    // OUT: sensitive area
                           );
 
-typedef const EVP_CIPHER *(*evpfunc)(void);
+const EVP_CIPHER *GetEVPCipher(TPM_ALG_ID    algorithm,       // IN
+			       UINT16        keySizeInBits,   // IN
+			       TPM_ALG_ID    mode,            // IN
+			       const BYTE   *key,             // IN
+			       BYTE         *keyToUse,        // OUT same as key or stretched key
+			       UINT16       *keyToUseLen      // IN/OUT
+			      );
+
+TPM_RC DoEVPGetIV(
+                  EVP_CIPHER_CTX    *ctx,    // IN: required context
+                  unsigned char     *iv,     // IN: pointer to buffer for IV
+                  size_t             iv_len  // IN: size of the buffer
+                  );
 
-evpfunc GetEVPCipher(TPM_ALG_ID    algorithm,       // IN
-                     UINT16        keySizeInBits,   // IN
-                     TPM_ALG_ID    mode,            // IN
-                     const BYTE   *key,             // IN
-                     BYTE         *keyToUse,        // OUT same as key or stretched key
-                     UINT16       *keyToUseLen      // IN/OUT
-                     );
 #endif
 
 #if USE_OPENSSL_FUNCTIONS_EC
@@ -114,4 +119,16 @@ InitOpenSSLRSAPrivateKey(OBJECT     *rsaKey,   // IN
 
 #endif // USE_OPENSSL_FUNCTIONS_RSA
 
+#if USE_OPENSSL_FUNCTIONS_SSKDF
+LIB_EXPORT UINT16
+OSSLCryptKDFe(TPM_ALG_ID   hashAlg,  // IN: hash algorithm used in HMAC
+	      TPM2B*       Z,        // IN: Z
+	      const TPM2B* label,    // IN: a label value for the KDF
+	      TPM2B*       partyUInfo,  // IN: PartyUInfo
+	      TPM2B*       partyVInfo,  // IN: PartyVInfo
+	      UINT32       sizeInBits,  // IN: size of generated key in bits
+	      BYTE*        keyStream    // OUT: key buffer
+	     );
+#endif // USE_OPENSSL_FUNCTIONS_SSKDF
+
 #endif  /* HELPERS_FP_H */
