@@ -38,10 +38,7 @@
 
 #include <config.h>
 
-#define _GNU_SOURCE
 #include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <stdbool.h>
 
 #ifndef LIB_EXPORT
@@ -59,15 +56,18 @@
 #include "tpm2/PlatformACT.h"
 #include "tpm2/PlatformData.h"
 #include "tpm2/Volatile.h"
+#include "tpm2/crypto/openssl/ExpDCache_fp.h"
 
 #define TPM_HAVE_TPM2_DECLARATIONS
-#include "tpm12/tpm_nvfile.h" // TPM_NVRAM_Loaddata()
+#include "tpm_nvfile.h" // TPM_NVRAM_Loaddata()
 #include "tpm_error.h"
 #include "tpm_library_intern.h"
 #include "tpm_nvfilename.h"
 
 extern BOOL      g_inFailureMode;
 static BOOL      reportedFailureCommand;
+static char     *g_profile;
+static TPM_BOOL  g_wasManufactured;
 
 /*
  * Check whether the main NVRAM file exists. Return TRUE if it doesn, FALSE otherwise
@@ -106,6 +106,7 @@ static TPM_RESULT TPM2_MainInit(void)
 
     g_inFailureMode = FALSE;
     reportedFailureCommand = FALSE;
+    g_wasManufactured = FALSE;
 
 #ifdef TPM_LIBTPMS_CALLBACKS
     struct libtpms_callbacks *cbs = TPMLIB_GetCallbacks();
@@ -130,15 +131,17 @@ static TPM_RESULT TPM2_MainInit(void)
 
     if (!has_cached_state) {
         if (!has_nvram_file) {
-            ret = _plat__NVEnable(NULL);
+            ret = _plat__NVEnable(NULL, 0);
             if (ret)
                 TPMLIB_LogTPM2Error(
                     "%s: _plat__NVEnable(NULL) failed: %d\n",
                     __func__, ret);
-            if (TPM_Manufacture(TRUE) < 0 || g_inFailureMode) {
+            if (TPM_Manufacture(TRUE, g_profile) < 0 || g_inFailureMode) {
                 TPMLIB_LogTPM2Error("%s: TPM_Manufacture(TRUE) failed or TPM in "
                                     "failure mode\n", __func__);
                 reportedFailureCommand = TRUE;
+            } else {
+                g_wasManufactured = TRUE;
             }
         }
     } else if (!has_nvram_loaddata_callback) {
@@ -169,6 +172,10 @@ static void TPM2_Terminate(void)
     TPM_TearDown();
 
     _rpc__Signal_PowerOff();
+    ExpDCacheFree();
+
+    free(g_profile);
+    g_profile = NULL;
 }
 
 static TPM_RESULT TPM2_Process(unsigned char **respbuffer, uint32_t *resp_size,
@@ -186,7 +193,8 @@ static TPM_RESULT TPM2_Process(unsigned char **respbuffer, uint32_t *resp_size,
     if (cbs->tpm_io_getlocality) {
         TPM_MODIFIER_INDICATOR locty;
 
-        locality = cbs->tpm_io_getlocality(&locty, 0);
+        /* called function is trusted and must return valid value */
+        cbs->tpm_io_getlocality(&locty, 0);
 
         locality = locty;
     }
@@ -355,8 +363,8 @@ static char *TPM2_GetInfo(enum TPMLIB_InfoFlags flags)
     const char *tpmspec =
     "\"TPMSpecification\":{"
         "\"family\":\"2.0\","
-        "\"level\":" STRINGIFY(SPEC_LEVEL_NUM) ","
-        "\"revision\":" STRINGIFY(SPEC_VERSION)
+        "\"level\":" STRINGIFY(TPM_SPEC_LEVEL_NUM) ","
+        "\"revision\":" STRINGIFY(TPM_SPEC_VERSION)
     "}";
     const char *tpmattrs_temp =
     "\"TPMAttributes\":{"
@@ -366,13 +374,57 @@ static char *TPM2_GetInfo(enum TPMLIB_InfoFlags flags)
     "}";
     const char *tpmfeatures_temp =
     "\"TPMFeatures\":{"
-        "\"RSAKeySizes\":[%s]"
+        "\"RSAKeySizes\":[%s],"
+        "\"CamelliaKeySizes\":[%s]"
+#if ALG_SM4
+        ", \"SM4KeySizes\":[128]"
+#endif
     "}";
+    const char *runtimeAlgorithms_temp =
+    "\"RuntimeAlgorithms\":{"
+        "\"Implemented\":%s,"
+        "\"CanBeDisabled\":%s,"
+        "\"Enabled\":%s,"
+        "\"Disabled\":%s"
+    "}";
+    const char *runtimeCommands_temp =
+    "\"RuntimeCommands\":{"
+        "\"Implemented\":%s,"
+        "\"CanBeDisabled\":%s,"
+        "\"Enabled\":%s,"
+        "\"Disabled\":%s"
+    "}";
+    const char *runtimeAttributes_temp =
+    "\"RuntimeAttributes\":{"
+        "\"Implemented\":%s,"
+        "\"CanBeDisabled\":%s,"
+        "\"Enabled\":%s,"
+        "\"Disabled\":%s"
+    "}";
+    const char *tpmProfile_temp = "\"ActiveProfile\":%s";
+    const char *availableProfiles_temp =
+    "\"AvailableProfiles\":["
+        "%s%s%s"
+    "]";
     char *fmt = NULL, *buffer;
     bool printed = false;
     char *tpmattrs = NULL;
     char *tpmfeatures = NULL;
     char rsakeys[32];
+    char camelliakeys[16];
+    char *runtimeAlgos[RUNTIME_ALGO_NUM] = { NULL, };
+    char *runtimeCmds[RUNTIME_CMD_NUM] = { NULL, };
+    char *runtimeAttrs[RUNTIME_ATTR_NUM] = { NULL, };
+    enum RuntimeAlgorithmType rat;
+    enum RuntimeCommandType rct;
+    enum RuntimeAttributeType rabt;
+    char *runtimeAlgorithms = NULL;
+    char *runtimeCommands = NULL;
+    char *runtimeAttributes = NULL;
+    char *profile = NULL;
+    const char *profileJSON;
+    char *availableProfiles = NULL;
+    char *tmp = NULL;
     size_t n;
 
     if (!(buffer = strdup("{%s%s%s}")))
@@ -381,19 +433,21 @@ static char *TPM2_GetInfo(enum TPMLIB_InfoFlags flags)
     if ((flags & TPMLIB_INFO_TPMSPECIFICATION)) {
         fmt = buffer;
         buffer = NULL;
-        if (asprintf(&buffer, fmt, "", tpmspec, "%s%s%s") < 0)
+        if (TPMLIB_asprintf(&buffer, fmt, "", tpmspec, "%s%s%s") < 0)
             goto error;
         free(fmt);
         printed = true;
     }
 
     if ((flags & TPMLIB_INFO_TPMATTRIBUTES)) {
+        uint32_t firmware_v1 = _plat__GetTpmFirmwareVersionHigh();
+
         fmt = buffer;
         buffer = NULL;
-        if (asprintf(&tpmattrs, tpmattrs_temp, FIRMWARE_V1) < 0)
+        if (TPMLIB_asprintf(&tpmattrs, tpmattrs_temp, firmware_v1) < 0)
             goto error;
-        if (asprintf(&buffer, fmt,  printed ? "," : "",
-                     tpmattrs, "%s%s%s") < 0)
+        if (TPMLIB_asprintf(&buffer, fmt,  printed ? "," : "",
+                             tpmattrs, "%s%s%s") < 0)
             goto error;
         free(fmt);
         printed = true;
@@ -408,10 +462,128 @@ static char *TPM2_GetInfo(enum TPMLIB_InfoFlags flags)
                      RSA_4096 ? ",4096" : "");
         if (n >= sizeof(rsakeys))
             goto error;
-        if (asprintf(&tpmfeatures, tpmfeatures_temp, rsakeys) < 0)
+        n = snprintf(camelliakeys, sizeof(camelliakeys), "%s%s%s",
+                     CAMELLIA_128 ? "128" : "",
+                     CAMELLIA_192 ? ",192" : "",
+                     CAMELLIA_256 ? ",256" : "");
+        if (n >= sizeof(camelliakeys))
             goto error;
-        if (asprintf(&buffer, fmt,  printed ? "," : "",
-                     tpmfeatures, "%s%s%s") < 0)
+        if (TPMLIB_asprintf(&tpmfeatures, tpmfeatures_temp,
+                            rsakeys, camelliakeys) < 0)
+            goto error;
+        if (TPMLIB_asprintf(&buffer, fmt,  printed ? "," : "",
+                            tpmfeatures, "%s%s%s") < 0)
+            goto error;
+        free(fmt);
+        printed = true;
+    }
+
+    if ((flags & TPMLIB_INFO_RUNTIME_ALGORITHMS)) {
+        fmt = buffer;
+        buffer = NULL;
+        for (rat = RUNTIME_ALGO_IMPLEMENTED; rat < RUNTIME_ALGO_NUM; rat++) {
+            runtimeAlgos[rat] = RuntimeAlgorithmPrint(&g_RuntimeProfile.RuntimeAlgorithm, rat);
+            if (!runtimeAlgos[rat])
+                goto error;
+        }
+        if (TPMLIB_asprintf(&runtimeAlgorithms, runtimeAlgorithms_temp,
+                            runtimeAlgos[RUNTIME_ALGO_IMPLEMENTED],
+                            runtimeAlgos[RUNTIME_ALGO_CAN_BE_DISABLED],
+                            runtimeAlgos[RUNTIME_ALGO_ENABLED],
+                            runtimeAlgos[RUNTIME_ALGO_DISABLED]) < 0)
+            goto error;
+        if (TPMLIB_asprintf(&buffer, fmt,  printed ? "," : "",
+                            runtimeAlgorithms, "%s%s%s") < 0)
+            goto error;
+        free(fmt);
+        printed = true;
+    }
+
+    if ((flags & TPMLIB_INFO_RUNTIME_COMMANDS)) {
+        fmt = buffer;
+        buffer = NULL;
+        for (rct = RUNTIME_CMD_IMPLEMENTED; rct < RUNTIME_CMD_NUM; rct++) {
+            runtimeCmds[rct] = RuntimeCommandsPrint(&g_RuntimeProfile.RuntimeCommands, rct);
+            if (!runtimeCmds[rct])
+                goto error;
+        }
+        if (TPMLIB_asprintf(&runtimeCommands, runtimeCommands_temp,
+                            runtimeCmds[RUNTIME_CMD_IMPLEMENTED],
+                            runtimeCmds[RUNTIME_CMD_CAN_BE_DISABLED],
+                            runtimeCmds[RUNTIME_CMD_ENABLED],
+                            runtimeCmds[RUNTIME_CMD_DISABLED]) < 0)
+            goto error;
+        if (TPMLIB_asprintf(&buffer, fmt,  printed ? "," : "",
+                            runtimeCommands, "%s%s%s") < 0)
+            goto error;
+        free(fmt);
+        printed = true;
+    }
+
+    if ((flags & TPMLIB_INFO_RUNTIME_ATTRIBUTES)) {
+        fmt = buffer;
+        buffer = NULL;
+        for (rabt = RUNTIME_ATTR_IMPLEMENTED; rabt < RUNTIME_ATTR_NUM; rabt++) {
+            runtimeAttrs[rabt] = RuntimeAttributesGet(&g_RuntimeProfile.RuntimeAttributes, rabt);
+            if (!runtimeAttrs[rabt])
+                goto error;
+        }
+        if (TPMLIB_asprintf(&runtimeAttributes, runtimeAttributes_temp,
+                            runtimeAttrs[RUNTIME_ATTR_IMPLEMENTED],
+                            runtimeAttrs[RUNTIME_ATTR_CAN_BE_DISABLED],
+                            runtimeAttrs[RUNTIME_ATTR_ENABLED],
+                            runtimeAttrs[RUNTIME_ATTR_DISABLED]) < 0)
+            goto error;
+        if (TPMLIB_asprintf(&buffer, fmt,  printed ? "," : "",
+                            runtimeAttributes, "%s%s%s") < 0)
+            goto error;
+        free(fmt);
+        printed = true;
+    }
+
+    if ((flags & TPMLIB_INFO_ACTIVE_PROFILE) &&
+        (profileJSON = RuntimeProfileGetJSON(&g_RuntimeProfile))) {
+        fmt = buffer;
+        buffer = NULL;
+        if (TPMLIB_asprintf(&profile, tpmProfile_temp, profileJSON) < 0)
+            goto error;
+        if (TPMLIB_asprintf(&buffer, fmt, printed ? "," : "",
+                            profile, "%s%s%s") < 0)
+            goto error;
+        free(fmt);
+        printed = true;
+    }
+
+    if ((flags & TPMLIB_INFO_AVAILABLE_PROFILES)) {
+        size_t idx = 0;
+        char *json = NULL;
+
+        fmt = buffer; // keep here in case of 'goto error'
+        buffer = NULL;
+
+        tmp = NULL;
+
+        while (RuntimeProfileGetByIndex(idx, &json) == TPM_RC_SUCCESS) {
+            if (TPMLIB_asprintf(&availableProfiles,
+                                idx > 0 ? tmp : availableProfiles_temp,
+                                idx > 0 ? "," : "",
+                                json,
+                                "%s%s%s") < 0) {
+                free(json);
+                goto error;
+            }
+
+            free(json);
+            free(tmp);
+            tmp = availableProfiles;
+            idx++;
+        }
+        availableProfiles = NULL;
+        if (TPMLIB_asprintf(&availableProfiles, tmp, "", "", "") < 0)
+            goto error;
+
+        if (TPMLIB_asprintf(&buffer, fmt, printed ? "," : "",
+                            availableProfiles, "%s%s%s") < 0)
             goto error;
         free(fmt);
         printed = true;
@@ -420,22 +592,32 @@ static char *TPM2_GetInfo(enum TPMLIB_InfoFlags flags)
     /* nothing else to add */
     fmt = buffer;
     buffer = NULL;
-    if (asprintf(&buffer, fmt, "", "", "") < 0)
+    if (TPMLIB_asprintf(&buffer, fmt, "", "", "") < 0)
         goto error;
 
+exit:
     free(fmt);
     free(tpmattrs);
     free(tpmfeatures);
+    free(profile);
+    for (rat = RUNTIME_ALGO_IMPLEMENTED; rat < RUNTIME_ALGO_NUM; rat++)
+        free(runtimeAlgos[rat]);
+    for (rct = RUNTIME_CMD_IMPLEMENTED; rct < RUNTIME_CMD_NUM; rct++)
+        free(runtimeCmds[rct]);
+    for (rabt = RUNTIME_ATTR_IMPLEMENTED; rabt < RUNTIME_ATTR_NUM; rabt++)
+        free(runtimeAttrs[rabt]);
+    free(runtimeAlgorithms);
+    free(runtimeCommands);
+    free(runtimeAttributes);
+    free(availableProfiles);
+    free(tmp);
 
     return buffer;
 
 error:
-    free(fmt);
     free(buffer);
-    free(tpmattrs);
-    free(tpmfeatures);
-
-    return NULL;
+    buffer = NULL;
+    goto exit;
 }
 
 static uint32_t tpm2_buffersize = TPM_BUFFER_MAX;
@@ -475,7 +657,7 @@ uint32_t TPM2_GetBufferSize(void)
  * successfully used by a TPM_INIT.
 */
 static TPM_RESULT TPM2_ValidateState(enum TPMLIB_StateType st,
-                                     unsigned int flags)
+                                     unsigned int flags LIBTPMS_ATTR_UNUSED)
 {
     TPM_RESULT ret = TPM_SUCCESS;
     TPM_RC rc = TPM_RC_SUCCESS;
@@ -659,6 +841,36 @@ static TPM_RESULT TPM2_SetState(enum TPMLIB_StateType st,
     return ret;
 }
 
+static TPM_RESULT TPM2_SetProfile(const char *profile)
+{
+    char *copyProfile = NULL;
+    TPM_RC rc;
+
+    if (_rpc__Signal_IsPowerOn())
+        return TPM_INVALID_POSTINIT;
+
+    if (profile) {
+        /* test the profile */
+        rc = RuntimeProfileTest(&g_RuntimeProfile, profile, true);
+        if (rc != TPM_RC_SUCCESS)
+            return TPM_FAIL;
+
+        copyProfile = strdup(profile);
+        if (!copyProfile)
+            return TPM_SIZE;
+    }
+
+    free(g_profile);
+    g_profile = copyProfile;
+
+    return TPM_SUCCESS;
+}
+
+static TPM_BOOL TPM2_WasManufactured(void)
+{
+    return g_wasManufactured;
+}
+
 const struct tpm_interface TPM2Interface = {
     .MainInit = TPM2_MainInit,
     .Terminate = TPM2_Terminate,
@@ -676,4 +888,6 @@ const struct tpm_interface TPM2Interface = {
     .ValidateState = TPM2_ValidateState,
     .SetState = TPM2_SetState,
     .GetState = TPM2_GetState,
+    .SetProfile = TPM2_SetProfile,
+    .WasManufactured = TPM2_WasManufactured,
 };
