@@ -11,20 +11,22 @@ int main(void)
     PERSISTENT_DATA pd;
 
     /* Check size of ppList that expands with new commands */
-#define PD_PP_LIST_EXP_SIZE 14
+    /* was 14 when COMPRESSED_LISTS was enabled */
+#define PD_PP_LIST_EXP_SIZE 17
     if (sizeof(pd.ppList) != PD_PP_LIST_EXP_SIZE) {
         fprintf(stderr,
-                "sizeof(PERSISTENT_DATA.ppList) does not have expecte size "
+                "sizeof(PERSISTENT_DATA.ppList) does not have expected size "
                 "of %u bytes but %zu bytes\n",
                 PD_PP_LIST_EXP_SIZE, sizeof(pd.ppList));
         return EXIT_FAILURE;
     }
 
     /* Check size of auditCommands that expands with new commands */
-#define PD_AUDIT_COMMANDS_EXP_SIZE 14
+    /* was 14 when COMPRESSED_LISTS was enabled */
+#define PD_AUDIT_COMMANDS_EXP_SIZE 17
     if (sizeof(pd.auditCommands) != PD_AUDIT_COMMANDS_EXP_SIZE) {
         fprintf(stderr,
-                "sizeof(PERSISTENT_DATA.auditCommands) does not have expecte size "
+                "sizeof(PERSISTENT_DATA.auditCommands) does not have expected size "
                 "of %u bytes but %zu bytes\n",
                 PD_AUDIT_COMMANDS_EXP_SIZE, sizeof(pd.auditCommands));
         return EXIT_FAILURE;
@@ -60,9 +62,9 @@ int main(void)
      * NVRAM fits on all architectures
      */
 #if RSA_4096
-# error Unsuported RSA key size
+# error Unsupported RSA key size
 #elif RSA_3072
-# define OBJECT_EXP_SIZE 2600
+# define OBJECT_EXP_SIZE 2608
 #elif RSA_2048
 # define OBJECT_EXP_SIZE 1896
 #endif
